import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Vector;

import widgets.ClientFrame;

import chat.Failure;
import chat.client.ChatClient;

/**
 * Lanceur d'un client de chat.
 *
 * @author davidroussel
 */
public class RunChatClient extends AbstractRunChat
{
	/**
	 * Hte sur lequel se trouve le serveur de chat
	 * @uml.property  name="host"
	 */
	private String host;

	/**
	 * Nom d'utilisateur  utiliser pour se connecter au serveur. Si le nom n'est pas fournit
	 * @uml.property  name="name"
	 */
	private String name;

	/**
	 * Flux d'entre sur lequel lire les messages taps par l'utilisateur
	 * @uml.property  name="in"
	 */
	private InputStream in;

	/**
	 * Flux de sortie sur lequel envoyer les messages vers l'utilisateur
	 * @uml.property  name="out"
	 */
	private OutputStream out;

	/**
	 * Indique si le client  crer est un GUI ou pas
	 * @uml.property  name="gui"
	 */
	private boolean gui;

	/**
	 * Ensemble des threads des clients. Il faudra attendre la fin de ces threads pour terminer l'excution principal.
	 * @uml.property  name="threadPool"
	 * @uml.associationEnd  multiplicity="(0 -1)" elementType="java.lang.Thread"
	 */
	private Vector<Thread> threadPool;

	/**
	 * Constructeur d'un lanceur de client d'aprs les arguments du programme
	 * principal
	 *
	 * @param args les arguments du programme principal
	 */
	protected RunChatClient(String[] args)
	{
		super(args);

		/*
		 * Initialisation des flux d'I/O utilisateur  null
		 * ils dpendrons du client  crer (console ou GUI)
		 */
		in = null;
		out = null;

		/*
		 * Initialisation du pool de thread des clients
		 */
		threadPool = new Vector<Thread>();
	}

	@Override
	protected void setAttributes(String[] args)
	{
		/*
		 * parsing des arguments communs aux clients et serveur -v | --verbose
		 * -p | --port : port  utiliser pour la serverSocket
		 */
		super.setAttributes(args);

		/*
		 * On met d'abord les attributs locaux  leur valeur par dfaut
		 */
		host = null;
		name = null;
		gui = true;

		/*
		 * parsing des arguments spcifique au client -h | --host : nom ou
		 * adresse IP du serveur -n | --name : nom d'utilisateur
		 * -g | --gui : pour lancer le client GUI
		 */
		for (int i = 0; i < args.length; i++)
		{
			if (args[i].equals("--host") || args[i].equals("-h"))
			{
				System.out.print("Setting host to : ");
				if (i < (args.length - 1))
				{
					// parse next arg for in port value
					host = args[++i];
					System.out.println(host);
				}
				else
				{
					System.out.println("nothing, invalid value");
				}
			}
			else if (args[i].equals("--name") || args[i].equals("-n"))
			{
				System.out.print("Setting user name to : ");
				if (i < (args.length - 1))
				{
					// parse next arg for in port value
					name = args[++i];
					System.out.println(name);
				}
				else
				{
					System.out.println("nothing, invalid value");
				}
			}
			if (args[i].equals("--gui") || args[i].equals("-g"))
			{
				gui = true;
			}
		}

		if (host == null) // on va chercher local host
		{
			try
			{
				host = InetAddress.getLocalHost().getHostName();
			}
			catch (UnknownHostException e)
			{
				System.err.println(Failure.NO_LOCAL_HOST);
				e.printStackTrace();
				System.exit(Failure.NO_LOCAL_HOST.toInteger());
			}
		}

		if (name == null) // on va chercher le nom de l'utilisateur
		{
			try
			{
				name = System.getenv("LOGNAME"); // Try LOGNAME on unix type
													// systems
			}
			catch (NullPointerException npe)
			{
				System.err.println("RunChatClient : no LOGNAME found, trying USERNAME");
				try
				{
					name = System.getenv("USERNAME"); // Try USERNAME on other
														// systems
				}
				catch (NullPointerException npe2)
				{
					System.err.println("RunChatClient : "
							+ Failure.NO_USER_NAME + " abort");
					System.exit(Failure.NO_USER_NAME.toInteger());
				}
			}
			catch (SecurityException se)
			{
				System.err.println("RunChatClient : " + Failure.NO_ENV_ACCESS
						+ " !");
				System.exit(Failure.NO_ENV_ACCESS.toInteger());
			}
		}
	}

	@Override
	protected void launch()
	{
		/*
		 * Create and Launch client
		 */
		if (verbose)
		{
			System.out.print("Creating client to " + host + " at port " + port
					+ " with verbose " + (verbose ? "on" : "off ... "));
		}

		Boolean commonRun;

		if (gui)
		{
			// lecture depuis la console
			in = System.in;
			// criture vers la console
			out = System.out;

			commonRun = null;
			/*
			 *  TODO A remplacer par :
			 * 	- Cration du ClientFrame (avec un tat d'excution commun  partager avec le ChatClient)
			 * 	- Cration de l'OutputStream  connecter sur l'InputStream du ClientFrame
			 * 	- Cration de l'InputStream  connecter sur l'OutputStream du ClientFrame
			 * 	- insertion du ClientFrame dans la boucle des evts GUI
			 * 	- Cration du thread pour faire tourner le ClientFrame et ajout au threadPool
			 * 	- Lancement du thread pour faire tourner le ClientFrame
			 */

			/*
			 * Cration de la ClientFrame
			 */

			ClientFrame cl = new ClientFrame(host,// hte du serveur
                    name,	// nom d'utilisateur
                    commonRun, // commonRun avec le GUI
                    verbose);// verbose
			
			/*
			 * Cration du flux de sortie vers l'utilisateur
			 * ( connecter au flux d'entre de la ClientFrame)
			 */

			
			try {
				out = new PipedOutputStream(cl.getIn());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			//in = cl.getIn();

			
			/*
			 * Cration du flux d'entre depuis l'utilisateur
			 * ( connecter au flux de sortie de la ClientFrame)
			 */
			try {
				
				in = new PipedInputStream(cl.getOut());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			

			/*
			 * Insertion de la frame dans la file des vnements GUI
			 */
//			EventQueue.invokeLater(new Runnable()
//			{
//				@Override
//				public void run()
//				{
//					try
//					{
//						frame.pack();
//						frame.setVisible(true);
//					}
//
//					catch (Exception e)
//					{
//						e.printStackTrace();
//					}
//				}
//			});

			/*
			 * Cration et lancement du thread de la frame
			 */
			Thread ct = new Thread(cl);
			threadPool.add(ct);
			ct.start();


		}
		else // client console
		{
			// lecture depuis la console
			in = System.in;
			// criture vers la console
			out = System.out;

			commonRun = null;
		}

		/*
		 * Lancement du ChatClient
		 */
		ChatClient client = new ChatClient(host,	// hte du serveur
		                                   port,	// port tcp
		                                   name,	// nom d'utilisateur
		                                   in,		// entres utilisateur
		                                   out,		// sorties utilisateur
		                                   commonRun, // commonRun avec le GUI
		                                   verbose);// verbose
		if (client.isReady())
		{
			if (verbose)
			{
				System.out.println("done");
			}

			Thread clientThread = new Thread(client);
			threadPool.add(clientThread);

			if (verbose)
			{
				System.out.print("Launching client ... ");
			}
			clientThread.start();
			if (verbose)
			{
				System.out.println("done");
			}

			// attente de l'ensemble des threads du threadPool pour terminer
			for (Thread t : threadPool)
			{
				try
				{
					t.join();
					if (verbose)
					{
						System.out.println("client thread end");
					}
				}
				catch (InterruptedException e)
				{
					System.err.println("interrupted");
					e.printStackTrace();
				}
			}
		}
		else
		{
			System.err.println("RunChatClient : client not ready, abort...");
			System.exit(3);
		}
	}

	/**
	 * Programme principal de lancement d'un client de chat
	 *
	 * @param args argument du programme
	 * <ul>
	 * 	<li>--host <host address> : set host to connect to</li>
	 * 	<li>--port <port number> : set host connection port</li>
	 * 	<li>--name <user name> : user name to use to connect</li>
	 * 	<li>--verbose : set verbose on</li>
	 * </ul>
	 */
	public static void main(String[] args)
	{

		RunChatClient client = new RunChatClient(args);

		client.launch();
	}

}
