package chat.client;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;

import chat.Failure;

/**
 * Classe Principale d'un client de chat. Instancie : - la socket pour
 * commnuniquer avec le serveur - le UserHandler pour traiter les messages de
 * l'utilisateur - le ServerHandler pour traiter les messages du serveur
 *
 * @author davidroussel
 */
public class ChatClient implements Runnable
{
	/**
	 * Nom d'utilisateur utilis pour se connecter
	 * @uml.property  name="userName"
	 */
	private String userName;

	/**
	 * Socket du client
	 * @uml.property  name="clientSocket"
	 */
	private Socket clientSocket;

	/**
	 * Flux d'entre depuis le serveur
	 * @uml.property  name="serverIn"
	 */
	private InputStream serverIn;

	/**
	 * Flux de sortie vers le serveur
	 * @uml.property  name="serverOut"
	 */
	private OutputStream serverOut;

	/**
	 * Ecrivain vers le flux de sortie vers le serveur. Utilis temporairement pour envoyer notre nom d'utilisateur au serveur
	 * @uml.property  name="serverOutPW"
	 */
	private PrintWriter serverOutPW;

	/**
	 * Flux d'entre depuis l'utilisateur
	 * @uml.property  name="userIn"
	 */
	private InputStream userIn;

	/**
	 * Flux de sortie vers l'utilisateur
	 * @uml.property  name="userOut"
	 */
	private OutputStream userOut;

	/**
	 * Handler des donnes en provenance du serveur
	 * @uml.property  name="serverHandler"
	 * @uml.associationEnd  aggregation="composite"
	 */
	private ServerHandler serverHandler = null;

	/**
	 * Handler des donnes en provenance de l'utilisateur
	 * @uml.property  name="userHandler"
	 * @uml.associationEnd  aggregation="composite"
	 */
	private UserHandler userHandler = null;

	/**
	 * Etat d'excution commun du   {@link #userHandler}   et du  {@link #serverHandler}  , lorsque l'un des deux Runnable se termine, il met commonRun  faux ce qui force l'autre  se terminer.
	 * @uml.property  name="commonRun"
	 */
	private Boolean commonRun;

	/**
	 * Etat du client. true si la socket ainsi que les diffrents flux d'entre/sortie ont t crs
	 * @uml.property  name="ready"
	 */
	private boolean ready;

	/**
	 * Affichage des messages de debug
	 * @uml.property  name="verbose"
	 */
	private boolean verbose;

	/**
	 * Constructeur d'un client de chat
	 *
	 * @param host l'adresse du serveur
	 * @param port le port  utiliser pour communiquer avec le serveur
	 * @param name le nom d'utilisateur utilis
	 * @param in le flux d'entre depuis l'utilisateur
	 * @param out le flux de sortie vers l'utilisateur
	 * @param l'tat d'excution commun avec un autre runnable. ou bien null
	 *            s'il n'y a pas d'autre runnable  synchroniser avec ceux
	 *            lancs dans le ChatClient
	 * @param verbose niveau de debug pour les messages
	 */
	public ChatClient(String host, int port, String name, InputStream in,
			OutputStream out, Boolean commonRun, boolean verbose)
	{
		this.verbose = verbose;
		userName = name;
		ready = false;

		// cration de la socket
		clientSocket = null;
		if (this.verbose)
		{
			System.err.print("ChatClient : Creating Socket ... ");
		}
		try
		{
			clientSocket = new Socket(host, port);
			if (this.verbose)
			{
				System.err.println("done");
			}
		}
		catch (UnknownHostException e)
		{
			System.err.println("ChatClient : " + Failure.UNKNOWN_HOST + " : "
					+ host);
			e.printStackTrace();
			System.exit(Failure.UNKNOWN_HOST.toInteger());
		}
		catch (IOException e)
		{
			System.err.println("ChatClient : " + Failure.CLIENT_CONNECTION
					+ " to: \"" + host + "\" at port \"" + port + "\"");
			e.printStackTrace();
			System.exit(Failure.CLIENT_CONNECTION.toInteger());
		}

		// Obtention du flux de sortie vers le serveur
		serverOut = null;
		if (this.verbose)
		{
			System.err
					.print("ChatClient : getting client output stream to server ... ");
		}
		try
		{
			serverOut = clientSocket.getOutputStream();
			if (this.verbose)
			{
				System.err.println("done");
			}

			if (serverOut == null)
			{
				System.err.println("ChatClient : null server out");
				System.exit(Failure.CLIENT_OUTPUT_STREAM.toInteger());
			}

		}
		catch (IOException e)
		{
			System.err.println("ChatClient : " + Failure.CLIENT_OUTPUT_STREAM);
			e.printStackTrace();
			System.exit(Failure.CLIENT_OUTPUT_STREAM.toInteger());
		}

		/*
		 * On a besoin d'un PrintWriter temporaire pour envoyer notre nom
		 * d'utilisateur au serveur afin qu'il puisse nous crer un thread ddi
		 *  notre traitement
		 */
		if (this.verbose)
		{
			System.err.print("ChatClient : sending name to server ... ");
		}
		serverOutPW = new PrintWriter(serverOut, true);

		/*
		 * Envoi du nom d'utilisateur au serveur afin qu'il cre un MainClient
		 * qui puisse tre trait dans un thread  part
		 */
		serverOutPW.println(userName);
		if (this.verbose)
		{
			System.err.println("done");
		}
		if (serverOutPW.checkError())
		{
			System.err.println("ChatClient : serverOutPw has errors");
		}

		// Obtention du flux d'entre depuis le serveur
		serverIn = null;
		if (this.verbose)
		{
			System.err
					.print("ChatClient : getting client input stream from Server ... ");
		}
		try
		{
			serverIn = clientSocket.getInputStream();
			if (this.verbose)
			{
				System.err.println("done");
			}
		}
		catch (IOException e)
		{
			System.err.println("ChatClient : " + Failure.CLIENT_INPUT_STREAM);
			e.printStackTrace();
			System.exit(Failure.CLIENT_INPUT_STREAM.toInteger());
		}

		// obtention des flux de l'utilisateur
		userIn = in;
		userOut = out;

		// Etat d'excution commun
		if (commonRun == null)
		{
			this.commonRun = new Boolean(true);
		}
		else
		{
			this.commonRun = commonRun;
		}

		// Cration du user handler
		userHandler = new UserHandler(userIn, serverOut, this.commonRun,
				verbose);

		// cration du server handler
		serverHandler = new ServerHandler(userName, serverIn, userOut,
				this.commonRun, verbose);

		ready = true;
	}

	/**
	 * Accs en lecture de l'tat du client
	 *
	 * @return the ready
	 * @uml.property name="ready"
	 */
	public boolean isReady()
	{
		return ready;
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run()
	{
		/*
		 * Tant que ce que l'on lit depuis l'utilisateur n'est pas null (avec un
		 * ctrl-D par exemple), on envoie ce que l'on a lu au serveur et on
		 * attends que celui ci nous rponde pour afficher ce qu'il nous envoie.
		 * On a donc deux boucles d'attente : d'une part l'utilisateur, d'autre
		 * part le serveur. Chaque boucle est donc traite dans son propre
		 * thread UserHandler traite les entres de l'utilisateur ServerHandler
		 * traite les entres du serveur et on attends la fin des deux threads
		 * pour terminer le client Les deux threads partagent une variable
		 * "commonRun" lorsque l'un des deux threads se termine il met cette
		 * vatiable  false. A chaque tour de boucle de chacun des threads ils
		 * consultent (de manire atomique) cette variable afin de savoir s'ils
		 * peuvent continuer
		 */

		Thread[] threads = new Thread[2];

		// Cration du thread du UserHandler
		threads[0] = new Thread(userHandler);

		// Cration du thread du ServerHandler
		threads[1] = new Thread(serverHandler);

		// Lancement des threads
		for (int i = 0; i < threads.length; i++)
		{
			threads[i].start();
		}

		for (int i = 0; i < threads.length; i++)
		{
			try
			{
				threads[i].join();
			}
			catch (InterruptedException e)
			{
				System.err.println("Join thread " + i + " interrupted");
			}
		}

		if (verbose)
		{
			System.err.println("ChatClient : All threads terminated");
		}

		cleanup();

	}

	/**
	 * Nettoyage du client : fermeture des flux d'entre/sortie et fermeture de
	 * la socket
	 */
	public void cleanup()
	{
		userHandler.cleanup();

		serverHandler.cleanup();

		// fermeture du flux temporaire de sortie vers le serveur
		if (verbose)
		{
			System.err.print("ChatClient : closing server output stream ... ");
		}
		serverOutPW.close();
		if (verbose)
		{
			System.err.println("done");
		}

		// fermeture de la socket
		if (verbose)
		{
			System.err.print("ChatClient : closing client socket ... ");
		}
		try
		{
			clientSocket.close();
			if (verbose)
			{
				System.err.println("done");
			}
		}
		catch (IOException e)
		{
			System.err.println("ChatClient : closing client socket failed");
			e.printStackTrace();
		}

	}
}
