package chat.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;

import chat.Failure;

/**
 * Server Handler. Classe s'occupant de lire le texte en provenance du serveur
 * et de le recopier sur le flux de sortie standard.
 *
 * @author davidroussel
 */
public class ServerHandler implements Runnable
{
	/**
	 * Le lecteur de flux d'entre en provenance du serveur
	 * @uml.property  name="serverInBR"
	 */
	private BufferedReader serverInBR;

	/**
	 * Ecrivain vers le flux de sortie vers l'utilisateur
	 * @uml.property  name="userOutPW"
	 */
	private PrintWriter userOutPW;

	/**
	 * Etat d'excution commun du ServerHandler et du   {@link UserHandler} 
	 * @uml.property  name="commonRun"
	 */
	private Boolean commonRun;

	/**
	 * Affichage des messages de debug
	 * @uml.property  name="verbose"
	 */
	private boolean verbose;

	/**
	 * Constructeur d'un ServerHandler
	 * @param name notre nom d'utilisateur sur le serveur
	 * @param in le flux d'entre en provenance du serveur
	 * @param out le flux de sortie vers l'utilisateur
	 * @param commonRun l'tat d'excution commun du {@link ServerHandler} et du
	 *            {@link UserHandler}
	 * @param verbose affichage des messages de debug
	 */
	public ServerHandler(String name, InputStream in, OutputStream out, Boolean commonRun, boolean verbose)
	{
		this.verbose = verbose;

		if (in != null)
		{
			if (this.verbose)
			{
				System.err.print("ServerHandler : creating server input reader ... ");
			}

			serverInBR = new BufferedReader(new InputStreamReader(in));

			if (this.verbose)
			{
				System.err.println("done");
			}
		}
		else
		{
			System.err.println("ServerHandler : " + Failure.CLIENT_INPUT_STREAM);
			System.exit(Failure.CLIENT_INPUT_STREAM.toInteger());
		}

		if (out != null)
		{
			if (verbose)
			{
				System.err.print("ServerHandler : creating user output writer ... ");
			}

			userOutPW = new PrintWriter(out);

			if (verbose)
			{
				System.err.println("done");
			}
		}
		else
		{
			System.err.println("ServerHandler : " + Failure.USER_OUTPUT_STREAM);
			System.exit(Failure.USER_OUTPUT_STREAM.toInteger());
		}

		if (commonRun != null)
		{
			this.commonRun = commonRun;
		}
		else
		{
			System.err.println("ServerHandler : null common run "
					+ Failure.OTHER);
			System.exit(Failure.OTHER.toInteger());
		}
	}

	/**
	 * Excution d'un ServerHandler.
	 * coute les entres en provenance du serveur et les envoient sur
	 * la sortie vers l'utilisateur
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run()
	{
		String serverInput = null;
		while (commonRun.booleanValue())
		{
			try
			{
				// lecture d'une ligne du serveur
				serverInput = serverInBR.readLine();
			}
			catch (IOException e)
			{
				System.err.println("ServerHandler : I/O error reading server");
				e.printStackTrace();
				break;
			}

			if (serverInput != null)
			{
				// Affichage vers l'utilisateur
				userOutPW.println(serverInput);
				if (userOutPW.checkError())
				{
					System.err.println("Serverhandler : userOutPw has errors");
				}
			}
			else
			{
				if (verbose)
				{
					System.err.println("ServerHandler : null input read");
				}
				break;
			}
		}

		if (commonRun.booleanValue())
		{
			if (verbose)
			{
				System.err.print("ServerHandler : changing run state at the end ... ");
			}
			synchronized (commonRun)
			{
				commonRun = Boolean.FALSE;
			}
			if (verbose)
			{
				System.err.println("done");
			}
		}
	}

	/**
	 * Fermeture des flux
	 */
	public void cleanup()
	{
		// fermeture du lecteur de flux d'entre du serveur
		if (verbose)
		{
			System.err.print("UserHandler : closing server input stream reader ... ");
		}
		try
		{
			serverInBR.close();
			if (verbose)
			{
				System.err.println("done");
			}
		}
		catch (IOException e)
		{
			System.err.println("UserHandler : closing server input stream reader failed");
			e.printStackTrace();
		}

		// fermeture de l'crivain vers le flux de sortie vers l'utilisateur
		if (verbose)
		{
			System.err.print("UserHandler : closing user output print writer ... ");
		}
		userOutPW.close();
		if (verbose)
		{
			System.err.println("done");
		}
	}
}
