package chat.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;

import chat.Failure;
import chat.Vocabulary;

/**
 * User Handler Classe s'occupant de rcuprer ce que tape l'utilisateur et de
 * l'envoyer au serveur de chat
 *
 * @author davidroussel
 */
public class UserHandler implements Runnable
{
	/**
	 * Lecteur du flux d'entre depuis l'utilisateur
	 * @uml.property  name="userInBR"
	 */
	private BufferedReader userInBR;

	/**
	 * Ecrivain vers le flux de sortie vers le serveur
	 * @uml.property  name="serverOutPW"
	 */
	private PrintWriter serverOutPW;

	/**
	 * Etat d'excution commun du UserHandler et du   {@link ServerHandler} 
	 * @uml.property  name="commonRun"
	 */
	private Boolean commonRun;

	/**
	 * Niveau de debug
	 * @uml.property  name="verbose"
	 */
	private boolean verbose;

	/**
	 * Constructeur d'un UserHandler
	 *
	 * @param in Le flux d'entre de l'utilisateur pour les entres
	 *            utilisateur
	 * @param out le flux de sortie vers le serveur
	 * @param commonRun l'tat d'excution commun du {@link UserHandler} et du
	 *            {@link ServerHandler}
	 */
	public UserHandler(InputStream in, OutputStream out, Boolean commonRun, boolean verbose)
	{
		this.verbose = verbose;
		/*
		 * Cration du lecteur de flux d'entre de l'utilisateur
		 */
		if (in != null)
		{
			if (this.verbose)
			{
				System.err.print("UserHandler : creating user input buffered reader ... ");
			}

			userInBR = new BufferedReader(new InputStreamReader(in));

			if (this.verbose)
			{
				System.err.println("done");
			}
		}
		else
		{
			System.err.println("UserHandler : null input stream" + Failure.USER_INPUT_STREAM);
			System.exit(Failure.USER_INPUT_STREAM.toInteger());
		}

		/*
		 * Cration de l'crivain vers le flux de sortie vers le serveur
		 */
		if (out != null)
		{
			if (verbose)
			{
				System.err.print("UserHandler : creating server output print writer ... ");
			}

			serverOutPW = new PrintWriter(out, true);

			if (verbose)
			{
				System.err.println("done");
			}
		}
		else
		{
			System.err.println("UserHandler : null output stream" + Failure.CLIENT_OUTPUT_STREAM);
			System.exit(Failure.CLIENT_OUTPUT_STREAM.toInteger());
		}

		this.commonRun = commonRun;

	}

	/**
	 * Exction d'un UserHandler.
	 * coute les entres en provenance de l'utilisateur et les envoie
	 * dans le flux de sortie vers le serveur
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run()
	{
		String userInput = null;
		while (commonRun.booleanValue())
		{
			try
			{
				// Lecture d'une ligne en provenance de l'utilisateur
				userInput = userInBR.readLine();
			}
			catch (IOException e)
			{
				System.err.println("UserHandler : I/O error reading user");
				e.printStackTrace();
				break;
			}

			if (userInput != null)
			{
				// envoi du texte au serveur
				serverOutPW.println(userInput);
				if (serverOutPW.checkError())
				{
					System.err.println("UserHandler : serverOutPW has errors");
				}

				// on vrifie si la commande "bye" a t tape par l'utilisateur
				if (userInput.toLowerCase().equals(Vocabulary.byeCmd))
				{
					break;
				}
			}
			else // userInput == null
			{
				if (verbose)
				{
					System.err.println("UserHandler : null user input");
				}
				break;
			}
		}

		if (commonRun.booleanValue())
		{
			if (verbose)
			{
				System.err.print("UserHandler : changing run state at the end ... ");
			}
			synchronized (commonRun)
			{
				commonRun = Boolean.FALSE;
			}
			if (verbose)
			{
				System.err.println("done");
			}
		}
	}

	/**
	 * Fermeture des flux
	 */
	public void cleanup()
	{
		// fermeture du lecteur de flux d'entre de l'utilisateur
		if (verbose)
		{
			System.err.print("UserHandler : closing user input stream reader ... ");
		}
		try
		{
			userInBR.close();
			if (verbose)
			{
				System.err.println("done");
			}
		}
		catch (IOException e)
		{
			System.err.println("UserHandler : closing server input stream reader failed");
			e.printStackTrace();
		}

		// fermeture de l'crivain vers le flux de sortie vers le serveur
		if (verbose)
		{
			System.err.print("UserHandler : closing server output print writer ... ");
		}
		serverOutPW.close();
		if (verbose)
		{
			System.err.println("done");
		}
	}
}
