package chat.server;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.Vector;

import chat.Failure;

/**
 * Classe du serveur de chat Chaque message de chaque client doit tre renvoy 
 * tous autres clients
 * 
 * @author davidroussel
 */
public class ChatServer implements Runnable
{
	/**
	 * La socket serveur
	 * @uml.property  name="serverSocket"
	 */
	private ServerSocket serverSocket;

	/**
	 * Le port par dfaut utilis
	 */
	public final static int DEFAULTPORT = 1394;

	/**
	 * Temps d'attente (en ms) par dfaut d'une connection d'un client. Au bout
	 * de ce temps une {@link SocketTimeoutException} est gnre et on peut
	 * choisir de recommencer  attendre (s'il reste des clients) ou bien
	 * arrter le serveur (s'il n'y a plus de clients)
	 */
	public final static int DEFAULTTIMEOUT = 1000;

	/**
	 * La liste des diffrents clients. Un client est constitu : <ul> <li>d'une   {@link Socket}  </li> <li>d'un nom :   {@link String}  </li> <li>d'un flux d'entre :   {@link BufferedReader}  </li> <li>d'un flux de sortie   {@link PrintWriter}  </li> </ul> Cette liste devra tre accde de manire synchrone par les diffrents threads des diffrents clients.
	 * @uml.property  name="clients"
	 * @uml.associationEnd  multiplicity="(0 -1)" ordering="true" aggregation="composite" inverse="chatServer:chat.server.Client"
	 */
	private Vector<Client> clients;

	/**
	 * Liste des handlers de chaque client
	 * @uml.property  name="handlers"
	 * @uml.associationEnd  multiplicity="(0 -1)" ordering="true" aggregation="composite" inverse="chatServer:chat.server.ClientHandler"
	 */
	private Vector<ClientHandler> handlers;

	/**
	 * Etat de verbosit
	 * @uml.property  name="verbose"
	 */
	private boolean verbose;

	/**
	 * Etat d'coute du serveur. Cet tat est vrai au dpart et passe  false lorsque le dernier client se dconnecte.
	 * @uml.property  name="listening"
	 */
	private boolean listening;

	/**
	 * Constructeur valu d'un serveur de chat. Celui ci initialise la
	 * {@link ServerSocket},
	 * 
	 * @param port le port sur lequel on coute les requtes
	 * @param verbose affiche les messages de dbug ou pas
	 * @param timeout temps d'attente de connection d'un client
	 * @throws IOException Si une erreur intervient lors de la cration de la
	 *             {@link ServerSocket}
	 */
	public ChatServer(int port, boolean verbose, int timeout)
			throws IOException
	{
		this.verbose = verbose;
		if (verbose)
		{
			System.err.print("ChatServer::ChatServer(" + port + ", verbose "
					+ (verbose ? "on" : "off") + ")");
		}
		serverSocket = new ServerSocket(port);
		if (serverSocket != null)
		{
			serverSocket.setSoTimeout(timeout);
		}

		clients = new Vector<Client>();
		handlers = new Vector<ClientHandler>();
	}

	/**
	 * Constructeur valu d'un serveur de chat. Celui ci initialise la
	 * {@link ServerSocket},
	 * 
	 * @param port le port sur lequel on coute les requtes
	 * @param verbose affiche les messages de dbug ou pas
	 * @throws IOException Si une erreur intervient lors de la cration de la
	 *             {@link ServerSocket}
	 */
	public ChatServer(int port, boolean verbose) throws IOException
	{
		this(port, verbose, DEFAULTTIMEOUT);
	}

	/**
	 * Constructeur par dfaut d'un serveur de chat. Celui ci initialise la
	 * {@link ServerSocket}, LE port utilis par dfaut est dfini par
	 * {@link #DEFAULTPORT}
	 * 
	 * @throws IOException Si une erreur intervient lors de la cration de la
	 *             {@link ServerSocket}
	 * @see #DEFAULTPORT
	 */
	public ChatServer() throws IOException
	{
		this(DEFAULTPORT, false);
	}

	/**
	 * Excution du serveur de chat : - On attend la connection d'un client -
	 * Lorsque celle ci se produit le client est trait dans un nouveau thread -
	 * Lorsqu'un client envoie un message au serveur, celui ci le rediffuse 
	 * l'ensemble des autres clients
	 * 
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run()
	{
		Vector<Thread> handlerThreads = new Vector<Thread>();
		listening = true;

		while (listening)
		{
			Socket clientSocket = null;
			String clientName = null;

			// acceptation de la socket du client
			try
			{
				// on attends ici une connection d'un nouveau client
				clientSocket = serverSocket.accept(); // --> IOException
				if (verbose)
				{
					System.err.println("client connection accepted");
				}

			}
			catch (SocketTimeoutException ste)
			{
				// on re-attends
				if (verbose)
				{
					System.err.println("Socket timeout, rewaiting ...");
				}
			}
			catch (IOException e)
			{
				System.err.println(Failure.SERVER_CONNECTION);
				e.printStackTrace();
				System.exit(Failure.SERVER_CONNECTION.toInteger());
			}

			if (clientSocket != null)
			{
				// rcupration du nom du client
				BufferedReader reader = null;
				if (verbose)
				{
					System.err
							.print("ChatServer : Creatingc client input stream to get client's name ... ");
				}
				try
				{
					reader = new BufferedReader(new InputStreamReader(
							clientSocket.getInputStream()));
					if (verbose)
					{
						System.err.println("done");
					}
				}
				catch (IOException e1)
				{
					System.err.println("ChatServer : "
							+ Failure.CLIENT_INPUT_STREAM);
					e1.printStackTrace();
					System.exit(Failure.CLIENT_INPUT_STREAM.toInteger());
				}
				if (reader != null)
				{
					if (verbose)
					{
						System.err
								.print("ChatServer : reading client's name : ");
					}
					try
					{
						// Lecture du nom du client
						clientName = reader.readLine();
						if (verbose)
						{
							System.err.println(clientName);
						}
					}
					catch (IOException e)
					{
						System.err.println("ChatServer : "
								+ Failure.NO_NAME_CLIENT);
						e.printStackTrace();
						System.exit(Failure.NO_NAME_CLIENT.toInteger());
					}

					/*
					 * On ne doit PAS fermer le client input stream car cela
					 * revient  fermer la socket
					 */
				}

				// Avant d'enregister cette connection dans l'ensemble des
				// clients il faut vrifier qu'aucun client ne porte le mme
				// nom
				if (searchClientByName(clientName) == null)
				{
					// Cration d'un nouveau client
					Client newClient = new Client(clientSocket, clientName,
							verbose);

					// Ajout du nouveau client  la liste des clients.
					synchronized (clients)
					{
						clients.add(newClient);
					}

					// Cration et lancement d'un handler pour ce client
					ClientHandler handler = new ClientHandler(this, newClient,
							clients);
					handlers.add(handler);
					Thread handlerThread = new Thread(handler);
					handlerThread.start();
					handlerThreads.add(handlerThread);
					
				}
				else
				// un client avec ce nom existe dj
				{
					// on notifie au client qu'il est refus
					try
					{
						PrintWriter out = new PrintWriter(
								clientSocket.getOutputStream(), true);
						out.println("server > Sorry another client already use the name "
								+ clientName);
						out.println("Hit ^D to close your client and try another name");
						out.close();
					}
					catch (IOException e)
					{
						System.err.println("ChatServer : "
								+ Failure.CLIENT_OUTPUT_STREAM);
						e.printStackTrace();
					}
				}

				/*
				 * Lorsqu'un ClientHandler se termine il lance la mthode
				 * cleanup qui lorqu'il n'y a plus aucun thread modifie la
				 * valeur de "listening"  false
				 */
			}
		} // while listening

		// attente de la fin de tous les threads de ClientHandler
		for (Thread t : handlerThreads)
		{
			try
			{
				t.join();
			}
			catch (InterruptedException e)
			{
				System.err
						.println("ChatServer::run : Client handlers join interrupted");
				e.printStackTrace();
			}
		}
		if (verbose)
		{
			System.err
					.println("ChatServer::run : all client handlers terminated");
		}

		handlerThreads.clear();
		handlers.clear();
		clients.clear();

		// Fermeture de la socket du serveur
		if (verbose)
		{
			System.err.print("ChatServer::run : Closing server socket ... ");
		}
		try
		{
			serverSocket.close();
			if (verbose)
			{
				System.err.println("done.");
			}
		}
		catch (IOException e)
		{
			System.err.println("Close serversocket Failed !");
			e.printStackTrace();
		}

	}

	/**
	 * Mthode invoque par les {@link ClientHandler}  la fin de leur excution
	 * pour ventuellement arrter le serveur lorsqu'il n'y a plus de clients
	 */
	protected synchronized void cleanup()
	{
		// s'il ne reste plus de threads on arrte la boucle
		int nbThreads = ClientHandler.getNbThreads();
		if (nbThreads <= 0)
		{
			listening = false;
			System.err.println("ChatServer::run : no more threads.");
		}
		else
		{
			System.err.println("ChatServer::run : still " + nbThreads
					+ " threads remaining ...");
		}
	}

	/**
	 * Recherche parmis les clients dj enregistrs un client portant le mme
	 * nom que l'argument
	 * 
	 * @param clientName le nom du client  rechercher parmis les clients dj
	 *            enregistrs
	 * @return le client recherch s'il existe ou bien null s'il n'existe pas
	 */
	protected Client searchClientByName(String clientName)
	{
		/*
		 * La consultation de la liste des clients  la recherche d'un nom doit
		 * tre atomique afin qu'aucun autre thread ne puisse modifier cette
		 * liste pendant qu'on la consulte : d'o le "synchronized"
		 */
		synchronized (clients)
		{
			for (Client c : clients)
			{
				if (c.getName().equals(clientName))
				{
					return c;
				}
			}
		}

		return null;
	}

}
