package chat.server;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.StringTokenizer;
import java.util.Vector;

import chat.Vocabulary;

/**
 * Classe utilise pour traiter chacune des connections des clients
 * 
 * @author davidroussel
 */
public class ClientHandler implements Runnable
{
	/**
	 * le ChatServer qui a lanc ce thread
	 * @uml.property  name="parent"
	 * @uml.associationEnd  aggregation="shared"
	 */
	private ChatServer parent;

	/**
	 * Le client principal de ce handler
	 * @uml.property  name="mainClient"
	 * @uml.associationEnd  aggregation="shared"
	 */
	private MainClient mainClient;

	/**
	 * Les autres clients relis au serveur.
	 * @uml.property  name="allClients"
	 * @uml.associationEnd  multiplicity="(1 -1)" ordering="true" aggregation="shared" inverse="clientHandler:chat.server.Client"
	 */
	private Vector<Client> allClients;

	/**
	 * Compteur d'instances du nombre de threads crs pour traiter les connections
	 * @uml.property  name="nbThreads"
	 */
	private static int nbThreads = 0;

	/**
	 * Constrcteur d'un handler de client
	 * 
	 * @param parent le {@link ChatServer} qui a lanc ce Runnable
	 * @param mainClient le client principal qu'il faut couter
	 * @param allClients les autres clients  qui il faut redistribuer ce
	 *            qu'envoie le client principal
	 */
	public ClientHandler(ChatServer parent, MainClient mainClient,
			Vector<Client> allClients)
	{
		this.parent = parent;
		this.mainClient = mainClient;
		this.allClients = allClients;
		nbThreads++;
	}

	/**
	 * Accesseur en lecture du nombre de ClientHandler en activit
	 * 
	 * @return the nbThreads
	 * @uml.property name="nbThreads"
	 */
	public static int getNbThreads()
	{
		return nbThreads;
	}

	/**
	 * Excution d'un handler de client. Consiste  lire une ligne du client
	 * jusqu' ce que l'on reoive la commande bye, ou qu'une IOException
	 * intervienne si le flux est coup
	 * 
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run()
	{
		boolean logout = false;
		String clientInput = null;
		StringBuffer broadcast = new StringBuffer(mainClient.getName());
		int broadcastHeader = broadcast.length();

		try
		{
			/*
			 * Attente d'une ligne de texte de la part d'un client (appel
			 * bloquant)
			 */
			while (!logout
					&& ((clientInput = mainClient.getIn().readLine()) != null))
			{
				// Affiche ce qui est reu par le client dans la console
				System.out.println(mainClient.getName() + " > " + clientInput);

				// Commande kick : Vocabulary.kickCmd
				// TODO  complter ...
				// on vrifie si on t banni par un super utilisateur
				// Si c'est le cas on quitte la boucle
				
				if (mainClient.isBanned())
				{
					broadcast.append(" banned");
					logout = true;
				}
				// on vrifie que le client ne veux pas nous quitter avec "bye"
				else if (!clientInput.toLowerCase().equals(Vocabulary.byeCmd))
				{
					broadcast.append(" > ");
					broadcast.append(clientInput);
				}
				else
				// le client veut se dloguer
				{
					broadcast.append(" logged out");
					logout = true;
				}
				
				//StringTockenizer pour reprer la commande kick
				StringTokenizer st = new StringTokenizer(clientInput.toLowerCase());
				String nameKick=null;
			    if (st.hasMoreTokens()) {
			    	//Nous vrifions si le premier tocken est la commande kick
			         if (st.nextToken().equals(Vocabulary.kickCmd) && st.hasMoreTokens())
			        	 //Si c'est le cas, le deuxime est forcment le pseudo
			        	 nameKick=st.nextToken();
			     }
			    
			    //Si nous avons un nom
			    if (nameKick != null)
			    {
			    	//Nous vrifions les privilges
			    	 if (mainClient.getName().equals(allClients.get(0).getName()))
			    	{
			    		//Nous recherchons le client parmis les connects
			    		Client ck = parent.searchClientByName(nameKick);
			    		
			    		if (ck!=null)
			    		{
			    			System.out.println(nameKick+" est kick du serveur par "+mainClient.getName());
			    			broadcast.append(" kick de "+nameKick+ "!");
			    			ck.setBanned(true); //Client bannis
			    		}
			    		else
			    		{
			    			broadcast.append("Pseudo introuvable.");
			    		}
			    	}
			    	 else
			    	 {
			    		 broadcast.append(" Cette instruction requiert les droits de super utilisateur."); 
			    	 }
			    }
			    


				/*
				 * rediffuse ce qui est reu du mainclient  tous les clients.
				 * allClients est un Vector qui est atomique donc a priori on a
				 * pas besoin du "synchronized (allClients)", Nanmoins ce
				 * synchronized permet de bloquer l'accs  l'ensemble des
				 * autres clients quand on diffuse le message de notre
				 * mainClient  tous les clients. Sans quoi on pourrait diffuser
				 * le message  un client, puis se faire interrompre par un
				 * autre client, puis diffuser le message  un autre client,
				 * etc. A vrifier ...
				 */
				synchronized (allClients)
				{
					for (Client c : allClients)
					{
						if (c.isReady())
						{
							PrintWriter out = c.getOut();
							out.println(broadcast.toString());
							if (out.checkError())
							{
								System.err.println("ClientHandler["
										+ mainClient.getName()
										+ "] : write to " + c.getName()
										+ " has errors");
							}
						}
						else
						{
							System.err.println("ClientHandler["
									+ mainClient.getName() + "]Client "
									+ c.getName() + " not ready");
						}
					}
				}

				// clean broadcast trailing characters
				broadcast.delete(broadcastHeader, broadcast.length());
			}
		}
		catch (IOException e)
		{
			System.err.println("ClientHandler[" + mainClient.getName()
					+ "] : received failed" + "Closing client " + this);
		}

		// remove current client from allClients (should be atomic)
		synchronized (allClients)
		{
			allClients.remove(mainClient);
		}
		// cleanup current client
		mainClient.cleanup();
		// dcrmentation du nombre de threads des clients
		nbThreads--;
		// Nettoyage du ChatServer parent (qui pourra evt s'arrter s'il n'y a
		// plus de clients)
		parent.cleanup();
	}

}
