package chat.server;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.Socket;

/**
 * Classe stockant les caractristiques d'un client principal : un client
 * principal est le client trait dans chacun des threads du ChatServer
 * <ul>
 * <li>socket : {@link Socket} du client</li>
 * <li>name : nom du client</li>
 * <li>in : {@link BufferedReader} lisant le texte en provenance du client</li>
 * </ul>
 * 
 * @author davidroussel
 */
public class MainClient
{
	/**
	 * La socket du client
	 * @uml.property  name="clientSocket"
	 */
	protected Socket clientSocket;

	/**
	 * Le nom du client
	 * @uml.property  name="name"
	 */
	protected String name;

	/**
	 * le flux d'entre du client (celui sur lequel on lit ce qui vient du client)
	 * @uml.property  name="inBR"
	 */
	protected BufferedReader inBR;

	/**
	 * Un Main client est "ready" lorsque sa clientSocket est non nulle et que l'on a russi  obtenir son input stream
	 * @uml.property  name="ready"
	 */
	protected boolean ready;

	/**
	 * Etat de bannissement du client. Ide : le premier utilisateur du serveur est considr comme le super-user. En consquence il a le privilge de pouvoir kicker des clients.
	 * @uml.property  name="banned"
	 */
	protected boolean banned;

	/**
	 * Niveau de debug pour les messages
	 * @uml.property  name="verbose"
	 */
	protected boolean verbose;

	/**
	 * @param socket the client's socket
	 * @param name the client's name
	 * @param niveau de debug pour les messages
	 */
	public MainClient(Socket socket, String name, boolean verbose)
	{
		clientSocket = socket;
		this.name = name;
		this.verbose = verbose;
		inBR = null;
		ready = false;

		if (socket != null)
		{
			if (verbose)
			{
				System.err.print("MainClient : Creating Input Stream ... ");
			}
			try
			{
				inBR = new BufferedReader(new InputStreamReader(
						socket.getInputStream()));
				ready = true;
				if (verbose)
				{
					System.err.println("done");
				}
			}
			catch (IOException e)
			{
				System.err
						.println("MainClient : unable to get client socket input stream");
				e.printStackTrace();
			}
		}
	}

	/**
	 * Accesseur en lecture du nom du client
	 * 
	 * @return the name
	 * @uml.property name="name"
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * Accesseur en lecture du flux d'entre du client
	 * 
	 * @return the input {@link BufferedReader}
	 */
	public BufferedReader getIn()
	{
		return inBR;
	}

	/**
	 * Accesseur en lecture de l'tat du client
	 * 
	 * @return the ready
	 * @uml.property name="ready"
	 */
	public boolean isReady()
	{
		return ready;
	}

	/**
	 * Accesseur en lecture de l'tat de banissement
	 * 
	 * @return l'tat de banissement
	 * @uml.property name="banned"
	 */
	public boolean isBanned()
	{
		return banned;
	}

	/**
	 * Accesseur en criture de l'tat de banissement
	 * 
	 * @param l'tat de banissement  mettre en place
	 * @uml.property name="banned"
	 */
	public void setBanned(boolean banned)
	{
		this.banned = banned;
	}

	/**
	 * Nettoyage d'un client principal : fermeture du flux d'entre et fermeture
	 * de sa socket.
	 */
	public void cleanup()
	{
		ready = false;
		if (verbose)
		{
			System.err.print("MainClient::cleanup : closing input stream ... ");
		}
		try
		{
			inBR.close();
			if (verbose)
			{
				System.err.println("done");
			}
		}
		catch (IOException e)
		{
			System.err
					.println("MainClient::cleanup : unable to close input stream");
			e.printStackTrace();
		}

		if (verbose)
		{
			System.err
					.print("MainClient::cleanup : closing client socket ... ");
		}
		try
		{
			clientSocket.close();
			if (verbose)
			{
				System.err.println("done");
			}
		}
		catch (IOException e)
		{
			System.err
					.println("MainClient::cleanup : unable to close client socket");
			e.printStackTrace();
		}
	}
}
